class Node():
    def __init__(self, initdata=None):
        self.data = initdata
        self.next = None
        self.prev = None

    def getData(self):
        return self.data

    def getNext(self):
        return self.next

    def getPrev(self):
        return self.prev

    def setData(self, newdata):
        self.data = newdata

    def setNext(self, newnext):
        self.next = newnext

    def setPrev(self, newprev):
        self.prev = newprev


# ======== 1 UnorderedList ========
# 用链表实现UnorderedList的如下方法：isEmpty, add, search, size, remove, append，index，pop，insert, __len__, __getitem__
# 用于列表字符串表示的__str__方法 (注：__str__里不要使用str(), 用repr()代替
# 用于切片的__getitem__方法
# 选做：UnorderedList(iterable) -> new UnorderedList initialized from iterable's items
# 选做：__eq__, __iter__
class UnorderedList():
    def __init__(self, argv=None):
        self.head = None
        if argv:
            for a in argv:
                self.append(a)

    def isEmpty(self) -> bool:
        return not bool(self.head)

    def add(self, item):
        temp = Node(item)
        temp.setNext(self.head)
        self.head = temp

    def search(self, item) -> bool:
        temp = self.head
        while temp:
            if temp.getData() == item:
                return True
            temp = temp.getNext()
        else:
            return False

    def size(self) -> int:
        count = 0
        temp = self.head
        while temp:
            count += 1
            temp = temp.getNext()
        return count

    def remove(self, item):
        previous = None
        current = self.head
        while current:
            if current.getData() == item:  # item found
                if not previous:  # first node
                    self.head = current.getNext()
                else:
                    previous.setNext(current.getNext())
                break
            previous = current
            current = current.getNext()

    def append(self, item):
        temp = Node(item)
        current = self.head
        if not current:  # append as first node
            self.head = temp
        else:  # append as last node
            while current.getNext():
                current = current.getNext()
            current.setNext(temp)

    # 搜索value，从下标start到stop，左闭右开
    def index(self, value, start=0, stop=None):
        if stop is None:
            stop = self.size()
        current = self.head
        i = 0
        while current:
            if i >= stop:
                break
            if start <= i < stop and current.getData() == value:
                return i
            current = current.getNext()
            i += 1

    def pop(self, pos=-1):
        if pos < 0:
            pos = self.size() + pos
        i = 0
        previous = None
        current = self.head
        while current:
            if i == pos:  # item found
                if not previous:  # first node
                    self.head = current.getNext()
                else:
                    previous.setNext(current.getNext())
                return current.getData()
            previous = current
            current = current.getNext()
            i += 1

    def insert(self, pos, item):
        temp = Node(item)
        i = 0
        previous = None
        current = self.head
        if not current:
            self.append(item)
            return
        while current:
            if i == pos:  # item found
                temp.setNext(current)
                if not previous:  # first node
                    self.head = temp
                else:
                    previous.setNext(temp)
                break
            previous = current
            current = current.getNext()
            i += 1

    def __len__(self) -> int:
        return self.size()

    def __str__(self) -> str:
        s = '['
        current = self.head
        if not current:
            return '[]'
        while current:
            s += repr(current.getData()) + ", "
            current = current.getNext()
        s = s[:-2] + ']'
        return s

    __repr__ = __str__

    def __getitem__(self, argv):
        #请参考：http://gis4g.pku.edu.cn/python3-slice-getitem/
        #可以不用处理负数
        if type(argv) is int:
            i = 0
            current = self.head
            while current:
                if i == argv:
                    return current.getData()
                current = current.getNext()
                i += 1
        else:
            t = UnorderedList()
            start = 0 if argv.start is None else argv.start
            stop = self.size() if argv.stop is None else argv.stop
            step = 1 if argv.step is None else argv.step
            for i in range(start, stop, step):
                t.append(self[i])
            return t

    ### 选做

    def __iter__(self):
        ptr = self.head
        while ptr:
            yield ptr.getData()
            ptr = ptr.getNext()

    def __eq__(self, other):
        if type(self) != type(other):
            return False

        if len(self) != len(other):
            return False

        return all(i == j for i, j in zip(self, other))

    # 实现__iter__后的__str__
    def __str__(self):
        return '[%s]' % ', '.join(map(repr, self))


# ======== 2 OrderedList ========
# 将OrderedList作为UnorderedList的子类来实现
# 实现ADT OrderedList的所有接口:
# add, remove, search, isEmpty, size, index, pop
# 注：不继承insert, append方法
class OrderedList(UnorderedList):
    insert = None
    append = None

    def add(self, item):
        temp = Node(item)
        previous = None
        current = self.head
        if not current:  # first item
            self.head = temp
            return
        while current:
            if current.getData() > item:
                temp.setNext(current)
                if not previous:  # first item
                    self.head = temp
                else:
                    previous.setNext(temp)
                break
            previous = current
            current = current.getNext()
        else:
            # last item
            previous.setNext(temp)


# ======== 3 ADT Stack & Queue ========
# 用链表实现ADT Stack与ADT Queue的所有接口
class Stack(UnorderedList):
    def push(self, item):
        self.add(item)

    def pop(self):
        return super().pop(0)

    def peek(self):
        return self.head.getData()


class Queue(UnorderedList):
    def enqueue(self, item):
        self.append(item)

    def dequeue(self):
        return self.pop(0)


# ======== 4 DoublyLinkedList ========
# 实现双向链表版本的UnorderedList，接口同ADT UnorderedList
# 在节点Node中增加prev变量，引用前一个节点
# 在UnorderedList中增加tail变量，引用列表中最后一个节点
# 增加getTail方法
class DoublyLinkedList(UnorderedList):
    def __init__(self, argv=None):
        self.tail = None
        super().__init__(argv)

    def getTail(self):
        return self.tail

    def add(self, item):
        temp = Node(item)
        temp.setNext(self.head)
        if self.head:
            self.head.setPrev(temp)
        else:
            self.tail = temp
        self.head = temp

    def remove(self, item):
        previous = None
        current = self.head
        while current:
            if current.getData() == item:  # item found
                ### 双链表
                next_node = current.getNext()
                if next_node:
                    next_node.setPrev(previous)
                else:  # last node
                    self.tail = previous
                ### 双链表

                if previous:
                    previous.setNext(next_node)
                else:  # first node
                    self.head = next_node
                break
            previous = current
            current = current.getNext()

    def append(self, item):
        temp = Node(item)
        temp.setPrev(self.tail)
        if self.tail:
            self.tail.setNext(temp)
        else:
            self.head = temp
        self.tail = temp

    def pop(self, pos=-1):
        if pos < 0:
            pos = self.size() + pos
        i = 0
        previous = None
        current = self.head
        while current:
            if i == pos:  # item found
                ### 双链表
                next_node = current.getNext()
                if next_node:
                    next_node.setPrev(previous)
                else:  # last node
                    self.tail = previous
                ### 双链表

                if previous:
                    previous.setNext(next_node)
                else:  # first node
                    self.head = next_node
                return current.getData()
            previous = current
            current = current.getNext()
            i += 1

    def insert(self, pos, item):
        temp = Node(item)
        i = 0
        previous = None
        current = self.head
        if not current:
            self.append(item)
            return
        while current:
            if i == pos:  # item found
                temp.setNext(current)
                temp.setPrev(previous)
                if not previous:  # first node
                    self.head = temp
                else:
                    previous.setNext(temp)
                if not current:  # last node
                    self.tail = temp
                else:
                    current.setPrev(temp)
                break
            previous = current
            current = current.getNext()
            i += 1

    def __getitem__(self, argv):
        res = super().__getitem__(argv)
        if isinstance(argv, slice) # isinstance(res, UnorderedList):
            res = DoublyLinkedList(res)
        return res

tmp=DoublyLinkedList(UnorderedList() for i in range(5))
print(type(tmp[0]))
